var fs = require('fs');
var request = require('request');
var querystring = require('querystring');
var async = require('async');

var mongoose = require('mongoose');

var connect = function () {
    mongoose.connect("mongodb://127.0.0.1:27017/sopcloud", {server: {socketOptions: {keepAlive: 1}}});
};
connect();
mongoose.connection.on('error', console.error.bind(console, 'connection error:'));
mongoose.connection.on('disconnected', connect);

var StatsSchema = new mongoose.Schema({
    date: { type: Date, default: null, index: {unique: false} },
    chid: { type: mongoose.Schema.Types.ObjectId, ref: 'Channel' },
    users: {type : Number, default : 0},
});


var Stats = mongoose.model('Stats', StatsSchema);
var AllStats = mongoose.model('AllStats', StatsSchema);

var overTrialRatio = 0.15

console.log("process.argv.length",process.argv)
var month = null
if (process.argv.length < 3) {
    console.log("usage: node report.js 2019-01");
    process.exit(-1);
} else {
    month = process.argv[2];
}
var monthStart = new Date(month + " GMT"+(-new Date().getTimezoneOffset()/60)+":00")
var monthEnd = new Date(month + " GMT"+(-new Date().getTimezoneOffset()/60)+":00")
monthEnd.setMonth(monthEnd.getMonth()+1)
console.log("#################### report monthStart:",monthStart);
console.log("#################### report monthEnd:",monthEnd);
dayMax=[]
header=[['Date', 'Viewers', 'Mark']]
AllStats.aggregate(
    {$match:{'date':{'$gt': monthStart, '$lt': monthEnd}}},
    {$group: {
        _id: "$date",
        "users" : { $sum: '$users' },
        "trialUsers" : { $sum: '$trialUsers' },
    }},
    { "$sort": { "_id": 1} },
    { $project: {_id: -1, date: '$_id', users: 1, trialUsers: 1}},

    function (err, statses ) {
        if(err) {
            logger.error(err);
            res.status(500).end();
        }
        else {
            // console.log("#################### report statses:",statses);
            tmpDayMax = statses[0]
            dayMSecs = 86400000
            for(var i=1; i<statses.length; i++) {
                if(statses[i].date < new Date(monthStart.getTime() + dayMSecs)) {
                    if(statses[i].users > tmpDayMax.users) {
			console.log(statses[i]);
                        tmpDayMax = statses[i]
                    }
                }
                else {
                    overTrial = tmpDayMax.trialUsers - parseInt(tmpDayMax.users * overTrialRatio)
                    if(overTrial>0) {
                        dayMax.push([tmpDayMax.date, tmpDayMax.users, tmpDayMax.trialUsers+"*", tmpDayMax.users + overTrial])
                    }
                    else {
                        dayMax.push([tmpDayMax.date, tmpDayMax.users, tmpDayMax.trialUsers, tmpDayMax.users])
                    }
                    tmpDayMax = statses[i];
                    dayMSecs = dayMSecs + 86400000
                }
            }
            overTrial = tmpDayMax.trialUsers - parseInt(tmpDayMax.users * overTrialRatio)
            if(overTrial>0) {
                dayMax.push([tmpDayMax.date, tmpDayMax.users, tmpDayMax.trialUsers+"*", tmpDayMax.users + overTrial])
            }
            else {
                dayMax.push([tmpDayMax.date, tmpDayMax.users, tmpDayMax.trialUsers, tmpDayMax.users])
            }

            console.log("\n\n========= Maximum Daily Online Viewers (", process.argv[2], ") =========");
            dayMax.sort(function(a, b) { return b[1] - a[1]; });
            for(var j=0; j<dayMax.length; j++) {
                console.log('"'+dayMax[j][0].toLocaleString('en-GB',{hour12: false, timeZoneName:'short'})+'","'+dayMax[j][1]+'","'+dayMax[j][2]+'","'+dayMax[j][3]+'"')
            }

            process.exit(0);
        }
    });
